@@ -18,20 +18,26 @@ import pytest
 
 from jupyter_core import paths
 from jupyter_core.paths import (
+    UF_HIDDEN,
+    _win32_restrict_file_to_user_ctypes,
+    exists,
     is_file_hidden,
     is_hidden,
+    issue_insecure_write_warning,
     jupyter_config_dir,
     jupyter_config_path,
     jupyter_data_dir,
     jupyter_path,
     jupyter_runtime_dir,
+    prefer_environment_over_user,
     secure_write,
 )
 
-from .mocking import darwin, linux
-
 pjoin = os.path.join
 
+macos = pytest.mark.skipif(sys.platform != "darwin", reason="only run on macos")
+windows = pytest.mark.skipif(sys.platform != "win32", reason="only run on windows")
+linux = pytest.mark.skipif(sys.platform != "linux", reason="only run on linux")
 
 xdg_env = {
     "XDG_CONFIG_HOME": "/tmp/xdg/config",
@@ -41,16 +47,13 @@ xdg_env = {
 xdg = patch.dict("os.environ", xdg_env)
 no_xdg = patch.dict(
     "os.environ",
-    {
-        "XDG_CONFIG_HOME": "",
-        "XDG_DATA_HOME": "",
-        "XDG_RUNTIME_DIR": "",
-    },
+    {},
 )
 
 no_config_env = patch.dict(
     "os.environ",
     {
+        "JUPYTER_PLATFORM_DIRS": "",
         "JUPYTER_CONFIG_DIR": "",
         "JUPYTER_DATA_DIR": "",
         "JUPYTER_RUNTIME_DIR": "",
@@ -58,16 +61,21 @@ no_config_env = patch.dict(
     },
 )
 
+use_platformdirs = patch.dict("os.environ", {"JUPYTER_PLATFORM_DIRS": "1"})
+
 jupyter_config_env = "/jupyter-cfg"
 config_env = patch.dict("os.environ", {"JUPYTER_CONFIG_DIR": jupyter_config_env})
 prefer_env = patch.dict("os.environ", {"JUPYTER_PREFER_ENV_PATH": "True"})
+prefer_user = patch.dict("os.environ", {"JUPYTER_PREFER_ENV_PATH": "False"})
 
 resetenv = patch.dict(os.environ)
 
 
 def setup_module():
     resetenv.start()
-    os.environ.pop("JUPYTER_PREFER_ENV_PATH", None)
+    # For these tests, default to preferring the user-level over environment-level paths
+    # Tests can override this preference using the prefer_env decorator/context manager
+    os.environ["JUPYTER_PREFER_ENV_PATH"] = "no"
 
 
 def teardown_module():
@@ -89,162 +97,217 @@ def test_envset():
             assert paths.envset(f"FOO_{v}")
         for v in false_values:
             assert not paths.envset(f"FOO_{v}")
-        assert not paths.envset("THIS_VARIABLE_SHOULD_NOT_BE_SET")
+        # Test default value is False
+        assert paths.envset("THIS_VARIABLE_SHOULD_NOT_BE_SET") is False
+        # Test envset returns the given default if supplied
+        assert paths.envset("THIS_VARIABLE_SHOULD_NOT_BE_SET", None) is None
 
 
-@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
+def test_config_dir():
+    config = jupyter_config_dir()
+    assert config == home_jupyter
+
+
+@macos
+@use_platformdirs
 def test_config_dir_darwin():
-    with darwin, no_config_env:
-        config = jupyter_config_dir()
-    assert config == home_jupyter
-
-    with darwin, config_env:
-        config = jupyter_config_dir()
-    assert config == jupyter_config_env
+    config = jupyter_config_dir()
+    assert config == realpath("~/Library/Preferences/Jupyter")
 
 
-@pytest.mark.skipif(sys.platform != "win32", reason="only run on windows")
+@windows
+@use_platformdirs
 def test_config_dir_windows():
-    with no_config_env:
-        config = jupyter_config_dir()
-    assert config == home_jupyter
+    config = jupyter_config_dir()
+    assert config == realpath(pjoin(os.environ.get("LOCALAPPDATA", ""), "Jupyter"))
 
+
+@linux
+@use_platformdirs
+def test_config_dir_linux():
+    config = jupyter_config_dir()
+    assert config == realpath("~/.config/jupyter")
+
+
+def test_config_env_legacy():
     with config_env:
         config = jupyter_config_dir()
-    assert config == jupyter_config_env
+        assert config == jupyter_config_env
 
 
-@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
-def test_config_dir_linux():
-    with linux, no_config_env:
+@use_platformdirs
+def test_config_env():
+    with config_env:
         config = jupyter_config_dir()
-    assert config == home_jupyter
-
-    with linux, config_env:
-        config = jupyter_config_dir()
-    assert config == jupyter_config_env
+        assert config == jupyter_config_env
 
 
+def test_data_dir_env_legacy():
+    data_env = "runtime-dir"
+    with patch.dict("os.environ", {"JUPYTER_DATA_DIR": data_env}):
+        data = jupyter_data_dir()
+        assert data == data_env
+
+
+@use_platformdirs
 def test_data_dir_env():
     data_env = "runtime-dir"
     with patch.dict("os.environ", {"JUPYTER_DATA_DIR": data_env}):
         data = jupyter_data_dir()
-    assert data == data_env
+        assert data == data_env
 
 
-@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
+@macos
+def test_data_dir_darwin_legacy():
+    data = jupyter_data_dir()
+    assert data == realpath("~/Library/Jupyter")
+
+
+@macos
+@use_platformdirs
 def test_data_dir_darwin():
-    with darwin:
-        data = jupyter_data_dir()
-    assert data == realpath("~/Library/Jupyter")
-
-    with darwin, xdg:
-        # darwin should ignore xdg
-        data = jupyter_data_dir()
-    assert data == realpath("~/Library/Jupyter")
+    data = jupyter_data_dir()
+    assert data == realpath("~/Library/Application Support/Jupyter")
 
 
-@pytest.mark.skipif(sys.platform != "win32", reason="only run on windows")
-def test_data_dir_windows():
+@windows
+def test_data_dir_windows_legacy():
     data = jupyter_data_dir()
     assert data == realpath(pjoin(os.environ.get("APPDATA", ""), "jupyter"))
 
+
+@windows
+@use_platformdirs
+def test_data_dir_windows():
+    data = jupyter_data_dir()
+    assert data == realpath(pjoin(os.environ.get("LOCALAPPDATA", ""), "Jupyter"))
+
+
+@linux
+def test_data_dir_linux_legacy():
+    with no_xdg:
+        data = jupyter_data_dir()
+        assert data == realpath("~/.local/share/jupyter")
+
     with xdg:
-        # windows should ignore xdg
         data = jupyter_data_dir()
-    assert data == realpath(pjoin(os.environ.get("APPDATA", ""), "jupyter"))
+        assert data == pjoin(xdg_env["XDG_DATA_HOME"], "jupyter")
 
 
-@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
+@linux
+@use_platformdirs
 def test_data_dir_linux():
-    with linux, no_xdg:
+    with no_xdg:
         data = jupyter_data_dir()
-    assert data == realpath("~/.local/share/jupyter")
+        assert data == realpath("~/.local/share/jupyter")
 
-    with linux, xdg:
+    with xdg:
         data = jupyter_data_dir()
-    assert data == pjoin(xdg_env["XDG_DATA_HOME"], "jupyter")
+        assert data == pjoin(xdg_env["XDG_DATA_HOME"], "jupyter")
 
 
+def test_runtime_dir_env_legacy():
+    rtd_env = "runtime-dir"
+    with patch.dict("os.environ", {"JUPYTER_RUNTIME_DIR": rtd_env}):
+        runtime = jupyter_runtime_dir()
+        assert runtime == rtd_env
+
+
+@use_platformdirs
 def test_runtime_dir_env():
     rtd_env = "runtime-dir"
     with patch.dict("os.environ", {"JUPYTER_RUNTIME_DIR": rtd_env}):
         runtime = jupyter_runtime_dir()
-    assert runtime == rtd_env
+        assert runtime == rtd_env
 
 
-@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
+@macos
+def test_runtime_dir_darwin_legacy():
+    runtime = jupyter_runtime_dir()
+    assert runtime == realpath("~/Library/Jupyter/runtime")
+
+
+@macos
+@use_platformdirs
 def test_runtime_dir_darwin():
-    with darwin:
-        runtime = jupyter_runtime_dir()
-    assert runtime == realpath("~/Library/Jupyter/runtime")
-
-    with darwin, xdg:
-        # darwin should ignore xdg
-        runtime = jupyter_runtime_dir()
-    assert runtime == realpath("~/Library/Jupyter/runtime")
+    runtime = jupyter_runtime_dir()
+    assert runtime == realpath("~/Library/Application Support/Jupyter/runtime")
 
 
-@pytest.mark.skipif(sys.platform != "win32", reason="only run on windows")
-def test_runtime_dir_windows():
+@windows
+def test_runtime_dir_windows_legacy():
     runtime = jupyter_runtime_dir()
     assert runtime == realpath(pjoin(os.environ.get("APPDATA", ""), "jupyter", "runtime"))
 
+
+@windows
+@use_platformdirs
+def test_runtime_dir_windows():
+    runtime = jupyter_runtime_dir()
+    assert runtime == realpath(pjoin(os.environ.get("LOCALAPPDATA", ""), "Jupyter", "runtime"))
+
+
+@linux
+def test_runtime_dir_linux_legacy():
+    with no_xdg:
+        runtime = jupyter_runtime_dir()
+        assert runtime == realpath("~/.local/share/jupyter/runtime")
+
     with xdg:
-        # windows should ignore xdg
         runtime = jupyter_runtime_dir()
-    assert runtime == realpath(pjoin(os.environ.get("APPDATA", ""), "jupyter", "runtime"))
+        assert runtime == pjoin(xdg_env["XDG_DATA_HOME"], "jupyter", "runtime")
 
 
-@pytest.mark.skipif(sys.platform == "win32", reason="does not run on windows")
+@linux
+@use_platformdirs
 def test_runtime_dir_linux():
-    with linux, no_xdg:
+    with no_xdg:
         runtime = jupyter_runtime_dir()
-    assert runtime == realpath("~/.local/share/jupyter/runtime")
+        assert runtime == realpath("~/.local/share/jupyter/runtime")
 
-    with linux, xdg:
+    with xdg:
         runtime = jupyter_runtime_dir()
-    assert runtime == pjoin(xdg_env["XDG_DATA_HOME"], "jupyter", "runtime")
+        assert runtime == pjoin(xdg_env["XDG_DATA_HOME"], "jupyter", "runtime")
 
 
 def test_jupyter_path():
     system_path = ["system", "path"]
     with no_config_env, patch.object(paths, "SYSTEM_JUPYTER_PATH", system_path):
         path = jupyter_path()
-    assert path[0] == jupyter_data_dir()
-    assert path[-2:] == system_path
+        assert path[0] == jupyter_data_dir()
+        assert path[-2:] == system_path
 
 
 def test_jupyter_path_user_site():
     with no_config_env, patch.object(site, "ENABLE_USER_SITE", True):
         path = jupyter_path()
 
-    # deduplicated expected values
-    values = list(
-        dict.fromkeys(
-            [
-                jupyter_data_dir(),
-                os.path.join(site.getuserbase(), "share", "jupyter"),
-                paths.ENV_JUPYTER_PATH[0],
-            ]
+        # deduplicated expected values
+        values = list(
+            dict.fromkeys(
+                [
+                    jupyter_data_dir(),
+                    os.path.join(site.getuserbase(), "share", "jupyter"),
+                    paths.ENV_JUPYTER_PATH[0],
+                ]
+            )
         )
-    )
-    for p, v in zip(path, values):
-        assert p == v
+        for p, v in zip(path, values):
+            assert p == v
 
 
 def test_jupyter_path_no_user_site():
     with no_config_env, patch.object(site, "ENABLE_USER_SITE", False):
         path = jupyter_path()
-    assert path[0] == jupyter_data_dir()
-    assert path[1] == paths.ENV_JUPYTER_PATH[0]
+        assert path[0] == jupyter_data_dir()
+        assert path[1] == paths.ENV_JUPYTER_PATH[0]
 
 
 def test_jupyter_path_prefer_env():
     with prefer_env:
         path = jupyter_path()
-    assert path[0] == paths.ENV_JUPYTER_PATH[0]
-    assert path[1] == jupyter_data_dir()
+        assert path[0] == paths.ENV_JUPYTER_PATH[0]
+        assert path[1] == jupyter_data_dir()
 
 
 def test_jupyter_path_env():
@@ -328,6 +391,36 @@ def test_jupyter_config_path_env():
     assert path[:2] == [pjoin("foo", "bar"), pjoin("bar", "baz")]
 
 
+def test_prefer_environment_over_user():
+    with prefer_env:
+        assert prefer_environment_over_user() is True
+
+    with prefer_user:
+        assert prefer_environment_over_user() is False
+
+    # Test default if environment variable is not set, and try to determine if we are in a virtual environment
+    os.environ.pop("JUPYTER_PREFER_ENV_PATH", None)
+
+    # base prefix differs, venv
+    with patch.object(sys, "base_prefix", "notthesame"):
+        assert prefer_environment_over_user() == paths._do_i_own(sys.prefix)
+
+    # conda
+    with patch.object(sys, "base_prefix", sys.prefix):
+        # in base env, don't prefer it
+        with patch.dict(os.environ, {"CONDA_PREFIX": sys.prefix, "CONDA_DEFAULT_ENV": "base"}):
+            assert not prefer_environment_over_user()
+        # in non-base env, prefer it
+        with patch.dict(os.environ, {"CONDA_PREFIX": sys.prefix, "CONDA_DEFAULT_ENV": "/tmp"}):
+            assert prefer_environment_over_user() == paths._do_i_own(sys.prefix)
+
+        # conda env defined, but we aren't using it
+        with patch.dict(
+            os.environ, {"CONDA_PREFIX": "/somewherelese", "CONDA_DEFAULT_ENV": "/tmp"}
+        ):
+            assert not prefer_environment_over_user()
+
+
 def test_is_hidden():
     with tempfile.TemporaryDirectory() as root:
         subdir1 = os.path.join(root, "subdir")
@@ -354,6 +447,14 @@ def test_is_hidden():
         assert not is_file_hidden(subdir56)
         assert not is_file_hidden(subdir56, os.stat(subdir56))
 
+        assert not is_file_hidden(os.path.join(root, "does_not_exist"))
+        subdir78 = os.path.join(root, "subdir7", "subdir8")
+        os.makedirs(subdir78)
+        assert not is_hidden(subdir78, root)
+        if hasattr(os, "chflags"):
+            os.chflags(subdir78, UF_HIDDEN)
+            assert is_hidden(subdir78, root)
+
 
 @pytest.mark.skipif(
     not (
@@ -412,6 +513,11 @@ def test_is_hidden_win32_pypy():
             assert "hidden files are not detectable on this system" in str(w[-1].message)
 
 
+@pytest.mark.skipif(sys.platform != "win32", reason="only runs on windows")
+def test_win32_restrict_file_to_user_ctypes(tmp_path):
+    _win32_restrict_file_to_user_ctypes(str(tmp_path))
+
+
 @pytest.mark.skipif(sys.platform != "win32", reason="only runs on windows")
 def test_secure_write_win32():
     def fetch_win32_permissions(filename):
@@ -479,3 +585,13 @@ def test_secure_write_unix():
             assert f.read() == "test 2"
     finally:
         shutil.rmtree(directory)
+
+
+def test_exists(tmpdir):
+    assert exists(str(tmpdir))
+
+
+def test_insecure_write_warning():
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", UserWarning)
+        issue_insecure_write_warning()
